package de.subway_surfers.vpr_app;

import Logik.Account;
import Logik.ElternAccount;
import Logik.Kind;
import Logik.MitarbeiterAccount;
import RestAPISchnittstelle.RestApiClient;
import javafx.collections.ListChangeListener;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.control.*;
import javafx.scene.layout.Background;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import com.google.gson.Gson;

import java.util.ArrayList;
import java.util.Random;

public class AccounterstellungMitarbeiter {

	private @FXML TextField accountname;
	private @FXML TextField email;
	private @FXML Label status;
	private @FXML Label einmalpw;
	private @FXML HBox kindanzeige;
	private @FXML GridPane kindDaten;
	private @FXML RadioButton typMitarbeiter;
	private @FXML RadioButton typEltern;
	private       ToggleGroup accountTyp;

	private ArrayList<Kind> kinder;

	public void initialize() {
		accountTyp = new ToggleGroup();
		accountTyp.getToggles().add(typEltern);
		accountTyp.getToggles().add(typMitarbeiter);
		einmalpw.setText(einmalPwGenerieren());

		kinder = new ArrayList<>();
	}

	public void onZurueck(ActionEvent actionEvent) {
		VerwaltungApplication.sceneWechseln("hauptmenue_mitarbeiter-view.fxml");
	}

	public void onAbmelden(ActionEvent actionEvent) {
		VerwaltungApplication.abmelden();
	}

	public void onTypMitarbeiter(ActionEvent actionEvent) {
		kindDaten.getChildren().clear();
	}

	public void onTypEltern(ActionEvent actionEvent) {
		Label ueberschrift = new Label();
		ueberschrift.setText("Daten des Kindes:");
		kindDaten.addRow(0);
		kindDaten.addColumn(0);
		kindDaten.addRow(1);
		kindDaten.add(ueberschrift,0,0);

		kindDaten.addRow(1);
		Label vname = new Label("Vorname:");
		kindDaten.add(vname, 0, 1);
		TextField vnameEingabe = new TextField();
		kindDaten.add(vnameEingabe, 1, 1);

		kindDaten.addRow(2);
		Label nname = new Label("Nachname:");
		kindDaten.add(nname, 0, 2);
		TextField nnameEingabe = new TextField();
		kindDaten.add(nnameEingabe, 1, 2);

		kindDaten.addRow(3);
		Label geburtstag = new Label("Geburtstag: ");
		kindDaten.add(geburtstag, 0, 3);
		DatePicker geburtstagEingabe = new DatePicker();
		geburtstagEingabe.setEditable(false);
		kindDaten.add(geburtstagEingabe, 1, 3);

		kindDaten.addRow(4);
		Button hinzufuegen = new Button("hinzufügen");
		kindDaten.add(hinzufuegen, 1, 4);

		kindDaten.addRow(5);
		hinzufuegen.setOnAction(e -> {
			Button neues = new Button(vnameEingabe.getText());

			boolean vnameGueltig = false;
			boolean gebGueltig = false;
			boolean nnameGueltig = false;

			if (!vnameEingabe.getText().equals("")) {
				vnameGueltig = true;
			}

			if (!nnameEingabe.getText().equals("")) {
				nnameGueltig = true;
			}

			if (!geburtstagEingabe.getEditor().getText().equals("")) {
				gebGueltig = true;
			}

			if (vnameGueltig && gebGueltig && nnameGueltig) {
				Kind kind = new Kind(nnameEingabe.getText(), vnameEingabe.getText(), 0);
				kinder.add(kind);
				kindanzeige.getChildren().add(neues);
				neues.setOnAction(a -> {
					((HBox) neues.getParent()).getChildren().remove(neues);
					kinder.remove(kind);
				});
				vnameEingabe.setText("");
				nnameEingabe.setText("");
				geburtstagEingabe.getEditor().setText("");
			}
		});
	}

	private String einmalPwGenerieren() {
		final int pwLaenge = 8;
		String pw = "";
		Random zufall = new Random();
		for (int i = 0; i < pwLaenge; i++) {
			pw += (char)zufall.nextInt('A', 'Z' + 1);
		}
		return pw;
	}

	private boolean eingabenGueltig() {
		return !accountname.getText().equals("") && !email.getText().equals("") && (typEltern.isSelected() || typMitarbeiter.isSelected());
	}

	public void onSpeichern(ActionEvent actionEvent) {
		if (eingabenGueltig()) {
			status.setText("Daten Erfolgreich gespeichert");
			einmalpw.setText(einmalPwGenerieren());


			//String json = String.format("{\"name\":\"%s\", \"email\":\"%s\", \"passwort\":\"%d\", \"rid\":\"%d\"}",
			//							accountname.getText(), email.getText(), einmalpw.getText().hashCode(), (typMitarbeiter.isSelected() ? 0:1));

			Account neuer;
			if (typEltern.isSelected()) {
				neuer = new ElternAccount(String.format("%d",einmalpw.getText().hashCode()), accountname.getText(), email.getText());
				for (Kind k : kinder) {
					((ElternAccount)neuer).getKinder().add(k);
				}
			}
			else if (typMitarbeiter.isSelected()) {
				neuer = new MitarbeiterAccount(String.format("%d",einmalpw.getText().hashCode()), accountname.getText(), email.getText());
			}
			else {
				return;
			}

			kinder.clear();

			Gson gson = new Gson();

			System.out.println(gson.toJson(neuer));
			RestApiClient api = new RestApiClient();
			api.post("Benutzer", gson.toJson(neuer));
			if (neuer instanceof ElternAccount) {
				for (Kind k : ((ElternAccount)neuer).getKinder()) {
					api.post("Kind", gson.toJson(k));
				}
			}
		}
		else {
			status.setText("Accounterstellung Fehlgeschlagen");
		}

	}

}
